@@ -117,13 +117,11 @@ struct write_window {
 };
 
 static pthread_mutex_t window_lock = PTHREAD_MUTEX_INITIALIZER;
-static pthread_rwlock_t window_fd_lock = PTHREAD_RWLOCK_INITIALIZER;
 static struct write_window window[NR_WINDOWS];
 
 static int
 evict_writes (int fd, uint64_t offset, size_t len)
 {
-  ACQUIRE_RDLOCK_FOR_CURRENT_SCOPE (&window_fd_lock);
   struct write_window oldest = { 0 };
 
   {
@@ -149,14 +147,10 @@ evict_writes (int fd, uint64_t offset, size_t len)
     window[NR_WINDOWS-1].offset = offset;
     window[NR_WINDOWS-1].len = len;
   }
-  /* Release window_lock here, but hold window_fd_lock since we don't
-   * want file_close -> remove_fd_from_window to close oldest.fd
-   */
+  /* Release lock here. */
 
   /* Evict the oldest window from the page cache (synchronously). */
   if (oldest.len > 0) {
-    int r;
-
     if (sync_file_range (oldest.fd, oldest.offset, oldest.len,
                          SYNC_FILE_RANGE_WAIT_BEFORE |
                          SYNC_FILE_RANGE_WRITE |
@@ -164,12 +158,9 @@ evict_writes (int fd, uint64_t offset, size_t len)
       nbdkit_error ("sync_file_range: cache=none: evicting oldest window: %m");
       return -1;
     }
-    r = posix_fadvise (oldest.fd, oldest.offset, oldest.len,
-                       POSIX_FADV_DONTNEED);
-    if (r != 0) {
-      errno = r;
+    if (posix_fadvise (oldest.fd, oldest.offset, oldest.len,
+                       POSIX_FADV_DONTNEED) == -1)
       nbdkit_debug ("posix_fadvise: POSIX_FADV_DONTNEED: (ignored): %m");
-    }
   }
 
   return 0;
@@ -181,7 +172,7 @@ evict_writes (int fd, uint64_t offset, size_t len)
 static void
 remove_fd_from_window (int fd)
 {
-  ACQUIRE_WRLOCK_FOR_CURRENT_SCOPE (&window_fd_lock);
+  ACQUIRE_LOCK_FOR_CURRENT_SCOPE (&window_lock);
   size_t i;
 
   for (i = 0; i < NR_WINDOWS; ++i)
@@ -694,10 +685,8 @@ file_open (int readonly)
     /* This is a hint so we ignore failures. */
 #ifdef HAVE_POSIX_FADVISE
     int r = posix_fadvise (h->fd, 0, 0, fadvise_mode);
-    if (r != 0) {
-      errno = r;
-      nbdkit_debug ("posix_fadvise: %s: (ignored): %m", h->name);
-    }
+    if (r == -1)
+      nbdkit_debug ("posix_fadvise: %s: %m (ignored)", h->name);
 #else
     nbdkit_debug ("fadvise is not supported");
 #endif
